@@ -46,17 +46,18 @@ def process_weights(ds,weights=None,target='ds'):
 
         # Regrid, if necessary (do nothing if the grids match up to within
         # floating-point precision)
-        if ((not ((ds.sizes['lat'] is weights.sizes['lat']) & (ds.sizes['lon'] is weights.sizes['lon']))) or 
+        if ((not ((ds.sizes['lat'] is weights.sizes['lat']) & (ds.sizes['lon'] == weights.sizes['lon']))) or 
             (not (np.allclose(ds.lat,weights.lat) & np.allclose(ds.lon,weights.lon)))):
-            if target is 'ds':
+            if target == 'ds':
                 print('regridding weights to data grid...')
                 # Create regridder to the [ds] coordinates
                 rgrd = xe.Regridder(weights,ds,'bilinear')
                 # Regrid [weights] to [ds] grids
                 weights = rgrd(weights)
 
-            elif target is 'weights':
-                raise KeyError('The '+target+' variable is not *yet* supported as a target for regridding. Please choose "ds" for now.')
+            elif target == 'weights':
+                raise NotImplementedError('The '+target+' variable is not *yet* supported as a target for regridding. Please choose "ds" for now.')
+                # This is because of lack of downstream capability right now... 
                 print('regridding data to weights grid...')
                 # Create regridder to the [weights] coordinates
                 rgrd = xe.Regridder(ds,weights,'bilinear')
@@ -99,9 +100,15 @@ def create_raster_polygons(ds,
     # of ocean pixels or something... 
           
     Returns:
-    a geopandas geodataframe containing a 'geometry' 
-    giving the pixel boundaries for each 'lat' / 'lon' 
-    pair
+    a dictionary containing:
+    - "gdf_pixels": a geopandas geodataframe containing a 
+                    'geometry' giving the pixel boundaries 
+                    for each 'lat' / 'lon' pair
+    - "source_grid": a dictionary containing the original
+                     lat and lon inputs under the keys
+                     "lat" and "lon" (just the 
+                     xr.DataArrays of those variables in
+                     the input ds)
                   
     Note: 
     'lat_bnds' and 'lon_bnds' can be created through the
@@ -121,7 +128,7 @@ def create_raster_polygons(ds,
     #breakpoint()
     # Subset by shapefile bounding box, if desired
     if subset_bbox is not None:
-        if type(subset_bbox) is gpd.geodataframe.GeoDataFrame:
+        if type(subset_bbox) == gpd.geodataframe.GeoDataFrame:
             # Using the biggest difference in lat/lon to make sure that the pixels are subset
             # in a way that the bounding box is fully filled out
             bbox_thresh = np.max([ds.lat.diff('lat').max(),ds.lon.diff('lon').max()])+0.1
@@ -135,7 +142,7 @@ def create_raster_polygons(ds,
             
     # Mask
     if mask is not None:
-        warnings.warn('Masking by grid not yet supported. Stay tuned...')
+        raise NotImplementedError('Masking by grid not yet supported. Stay tuned...')
         
     # Create dataset which has a lat/lon bound value for each individual pixel, 
     # broadcasted out over each lat/lon pair
@@ -183,8 +190,8 @@ def create_raster_polygons(ds,
     gdf_pixels['pix_idx'] = gdf_pixels.index.values
     
     # Add crs (normal lat/lon onto WGS84)
-    #gdf_pixels = gdf_pixels.set_crs("EPSG:4326")
-    gdf_pixels.crs = {'init':'EPSG:4326'}
+    gdf_pixels = gdf_pixels.set_crs("EPSG:4326")
+    #gdf_pixels.crs = {'init':'EPSG:4326'}
     
     # Save the source grid for further reference
     source_grid = {'lat':ds_bnds.lat,'lon':ds_bnds.lon}
@@ -247,20 +254,22 @@ def get_pixel_overlaps(gdf_in,pix_agg):
     # (using the EASE grid https://nsidc.org/data/ease)
     if np.all(gdf_in.total_bounds[[1,3]]>0):
         # If min/max lat are both in NH, use North grid
-        epsg_set = {'init':'EPSG:6931'}
+        #epsg_set = {'init':'EPSG:6931'} (change to below bc of depreciation of {'init':...} format in geopandas)
+        epsg_set = 'EPSG:6931'
     elif np.all(gdf_in.total_bounds[[1,3]]<0):
         # If min/max lat are both in SH, use South grid
-        epsg_set = {'init':'EPSG:6932'}
+        #epsg_set = {'init':'EPSG:6932'}
+        epsg_set = 'EPSG:6932'
     else:
         # Otherwise, use the global/temperate grid
-        epsg_set = {'init':'EPSG:6933'}
+        #epsg_set = {'init':'EPSG:6933'}
+        epsg_set = 'EPSG:6933'
     
     overlaps = gpd.overlay(gdf_in.to_crs(epsg_set),
                            pix_agg['gdf_pixels'].to_crs(epsg_set),
                            how='intersection')
     
     # Now, group by poly_idx (each polygon in the shapefile)
-    #(check if poly_idx exists in gdf_in first?)
     ov_groups = overlaps.groupby('poly_idx')
     
     # Calculate relative area of each overlap (so how much of the total 
@@ -296,10 +305,6 @@ def get_pixel_overlaps(gdf_in,pix_agg):
     if 'weights' in pix_agg['gdf_pixels'].columns:
         wm_out.weights = pix_agg['gdf_pixels'].weights
     
-    # Really, gdf_out should be its own class, with poly_idx, rel_area, 
-    # pix_idxs, and coords required; and the rest of the information the 
-    # stuff you keep
-    
     return wm_out
 
 
@@ -342,6 +347,8 @@ def aggregate(ds,wm):
     An xa.aggregated object with the aggregated variables 
     
     """
+    # Run ds through fix_ds (to fix lat/lon names, lon coords)
+    ds = fix_ds(ds)
 
     # Stack 
     ds = ds.stack(loc=('lat','lon'))
@@ -350,9 +357,14 @@ def aggregate(ds,wm):
     ds = subset_find(ds,wm.source_grid)
     
     # Set weights; or replace with ones if no additional weight information
-    if wm.weights is not 'nowghts':
+    #if wm.weights != 'nowghts':
+    if type(wm.weights) == pd.core.series.Series:
         weights = np.array([float(k) for k in wm.weights])
     else:
+        if wm.weights != 'nowghts':
+            warnings.warn('wm.weights is: \n '+print(wm.weights)+
+                            ', \n which is not a supported weight vector (in a pandas series) '+
+                            'or "nowghts" as a string. Assuming no weights are included...')
         weights = np.ones((len(wm.source_grid['lat'])))
     
     for var in ds.var():
@@ -367,12 +379,46 @@ def aggregate(ds,wm):
             for poly_idx in wm.agg.poly_idx:
                 # Get average value of variable over the polygon; weighted by 
                 # how much of the pixel area is in the polygon, and by (optionally)
-                # a separate gridded weight
-                if wm.agg.iloc[poly_idx,:].pix_idxs is not np.nan:
-                    wm.agg.loc[poly_idx,var] = [[((ds[var].isel(loc=wm.agg.iloc[poly_idx,:].pix_idxs)*
-                                                           normalize(np.squeeze(wm.agg.iloc[poly_idx,:].rel_area)*
-                                                             weights[wm.agg.iloc[poly_idx,:].pix_idxs])).
-                                                          sum('loc')).values]]
+                # a separate gridded weight. This if statement checks 
+                # - if the weights output for this polygon is just "np.nan", which 
+                #   indicates there's no overlap between the pixel grid and polygons
+                # - [this has been pushed down a few lines] or, if all the pixels in 
+                #   the grid have just nan values for this variable
+                # in both cases; the "aggregated variable" is just a vector of nans. 
+                if not np.isnan(wm.agg.iloc[poly_idx,:].pix_idxs).all():
+                    # Get the dimensions of the variable that aren't "loc" (location)
+                    other_dims = [k for k in np.atleast_1d(ds[var].dims) if k != 'loc']
+                    # Check first if any nans are "complete" (meaning that a pixel 
+                    # either has values for each step, or nans for each step - if
+                    # there are random nans within a pixel, throw a warning)
+                    if not xr.Dataset.equals(np.isnan(ds[var].isel(loc=wm.agg.iloc[poly_idx,:].pix_idxs)).any(other_dims),
+                                             np.isnan(ds[var].isel(loc=wm.agg.iloc[poly_idx,:].pix_idxs)).all(other_dims)):
+                        warnings.warn('One or more of the pixels in variable '+var+' have nans in them in the dimensions '+
+                                      ', '.join([k for k in ds[var].dims if k != 'loc'])+
+                                      '. The code can currently only deal with pixels for which the '+
+                                      '*entire* pixel has nan values in all dimensions, however there is currently no '+
+                                      ' support for data in which pixels have only some nan values. The aggregation '+
+                                      'calculation is likely incorrect.')
+
+                    if not np.isnan(ds[var].isel(loc=wm.agg.iloc[poly_idx,:].pix_idxs)).all(): 
+                        # Get relative areas for the pixels overlapping with this Polygon
+                        tmp_areas = np.squeeze(wm.agg.iloc[poly_idx,:].rel_area)
+                        # Replace overlapping pixel areas with nans if the corresponding pixel
+                        # is only composed of nans
+                        tmp_areas[np.array(np.isnan(ds[var].isel(loc=wm.agg.iloc[poly_idx,:].pix_idxs)).all(other_dims).values)] = np.nan
+                        # Calculate the normalized area+weight of each pixel (taking into account
+                        # nans)
+                        normed_areaweights = normalize(tmp_areas*weights[wm.agg.iloc[poly_idx,:].pix_idxs],drop_na=True)
+
+                        # Take the weighted average of all the pixel values to calculate the 
+                        # aggregated value for the shapefile
+                        wm.agg.loc[poly_idx,var] = [[((ds[var].isel(loc=wm.agg.iloc[poly_idx,:].pix_idxs)*
+                                                        normed_areaweights).
+                                                       sum('loc')).values]]
+
+                    else:
+                        wm.agg.loc[poly_idx,var] = [[(ds[var].isel(loc=0)*np.nan).values]]
+
                 else:
                     #breakpoint()
                     #wm.agg.loc[poly_idx,var] = [[np.array(np.nan)]]
